/*******************************************************************************
* FILE NAME: user_routines_fast.c <FRC VERSION>
*
* DESCRIPTION:
*  This file is where the user can add their custom code within the framework
*  of the routines below. 
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*
* OPTIONS:  Interrupts are disabled and not used by default.
*
*******************************************************************************/

#include <usart.h>
#include <adc.h>
#include <capture.h>
#include <timers.h>

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "tracker.h"
#include "beacon.h"

#include "user_routines.h"
#include "print_routines.h"

void IR_Sensor_Tracking(void);


/*** DEFINE USER VARIABLES AND INITIALIZE THEM HERE ***/


/*******************************************************************************
* FUNCTION NAME: InterruptVectorLow
* PURPOSE:       Low priority interrupt vector
* CALLED FROM:   nowhere by default
* ARGUMENTS:     none
* RETURNS:       void
* DO NOT MODIFY OR DELETE THIS FUNCTION 
*******************************************************************************/
#pragma code InterruptVectorLow = LOW_INT_VECTOR
void InterruptVectorLow (void)
{
  _asm
    goto InterruptHandlerLow  /*jump to interrupt routine*/
  _endasm
}


/*******************************************************************************
* FUNCTION NAME: InterruptHandlerLow
* PURPOSE:       Low priority interrupt handler
* If you want to use these external low priority interrupts or any of the
* peripheral interrupts then you must enable them in your initialization
* routine.  Innovation First, Inc. will not provide support for using these
* interrupts, so be careful.  There is great potential for glitchy code if good
* interrupt programming practices are not followed.  Especially read p. 28 of
* the "MPLAB(R) C18 C Compiler User's Guide" for information on context saving.
*
* Timer 0 is used by beacon.c
* Timer 1 is used by receiver.c
* Timer 2 is used by beacon.c
* Timer 4 is used to keep a clock
*
* CALLED FROM:   this file, InterruptVectorLow routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
#pragma code
#pragma interruptlow InterruptHandlerLow save=PROD /* You may want to save additional symbols. */

void InterruptHandlerLow ()     
{                               
  unsigned char Port_B;
  unsigned char Port_B_Delta;       
  static unsigned char Old_Port_B = 0xFF; // state of port b the last time
											// this function was called  
  static unsigned char	t100ms=0;
  static unsigned int	  t1sec=0;

  /*** Service Internal timer interrupts ***/

	if (INTCONbits.TMR0IF) // timer 0 Interrupt
	{
		INTCONbits.TMR0IF = 0; // clear the interrupt flag
  	Timer_0_Int_Handler(); // call the beacon interrupt handler
		// if used, call the timer 0 interrupt handler here
	}

	else if (PIR1bits.TMR1IF) // timer 1 interrupt - used by the IR receiver
	{
		PIR1bits.TMR1IF = 0; // clear the interrupt flag
	Timer_1_Int_Handler(); // call the timer 1 interrupt handler (in receiver.c)
	} 
 
  else if (PIR3bits.TMR4IF)	/* TIMER 4 INTERRUPT */
  {
    PIR3bits.TMR4IF = 0;	/* Clear Timer interrupt flag */
  }                             

  /*** Service External Interrupts ***/

	else if (INTCON3bits.INT2IF) // external interrupt 1
	{ 
		INTCON3bits.INT2IF = 0; // clear the interrupt flag
			// if used, call the interrupt 1 handler here
	}

	else if (INTCON3bits.INT3IF) // external interrupt 2 
	{
		INTCON3bits.INT3IF = 0; // clear the interrupt flag
			// if used, call the interrupt 2 handler here
	}

	else if (INTCONbits.RBIF) // external interrupts 3 through 6
	{
		Port_B = PORTB; // remove the "mismatch condition" by reading port b            
		INTCONbits.RBIF = 0; // clear the interrupt flag
		Port_B_Delta = Port_B ^ Old_Port_B; // determine which bits have changed
		Old_Port_B = Port_B; // save a copy of port b for next time around
	 
		if(Port_B_Delta & 0x10) // did external interrupt 3 change state? - IR sensor 1
		{
			Int_3_Handler(Port_B & 0x10 ? 1 : 0); // call the interrupt 3 handler (in receiver.c)
		}

		if(Port_B_Delta & 0x20) // did external interrupt 4 change state? - IR sensor 2
		{
			Int_4_Handler(Port_B & 0x20 ? 1 : 0); // call the interrupt 4 handler (in receiver.c)
		}

		if(Port_B_Delta & 0x40) // did external interrupt 5 change state? - IR sensor 3
		{
			Int_5_Handler(Port_B & 0x40 ? 1 : 0); // call the interrupt 5 handler (in receiver.c)
		}
		
		if(Port_B_Delta & 0x80) // did external interrupt 6 change state? - IR sensor 4
		{
			Int_6_Handler(Port_B & 0x80 ? 1 : 0); // call the interrupt 6 handler (in receiver.c)

		}
	}
}


/*******************************************************************************
* FUNCTION NAME: User_Autonomous_Code
* PURPOSE:       Execute user's code during autonomous robot operation.
* You should modify this routine by adding code which you wish to run in
* autonomous mode.  It will be executed every program loop, and not
* wait for or use any data from the Operator Interface.
* CALLED FROM:   main.c file, main() routine when in Autonomous mode
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void User_Autonomous_Code(void)
{
  while (autonomous_mode)   /* DO NOT CHANGE! */
  {
    if (statusflag.NEW_SPI_DATA)      /* 26.2ms loop area */
    {
        Getdata(&rxdata);   /* DO NOT DELETE, or you will be stuck here forever! */

        /* Add your own autonomous code here. */

        Generate_Pwms(pwm13,pwm14,pwm15,pwm16);

        Putdata(&txdata);   /* DO NOT DELETE, or you will get no PWM outputs! */
    }
  }
}


/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Local_IO
* PURPOSE:       Execute user's realtime code.
* You should modify this routine by adding code which you wish to run fast.
* It will be executed every program loop, and not wait for fresh data 
* from the Operator Interface.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Local_IO(void)
{
  static char toggle=-1;

  PrintString((char *) "rc_dig_in_06 = ");
  PrintDecimal(rc_dig_in_06);
  PrintCR();

  if (!rc_dig_in08)  /* Override tracker control of sensor servos and force servos to center position */
  {
    LEFT_TRACKER_SERVO  = 127;
	  RIGHT_TRACKER_SERVO = 127;
    if (toggle != 2)
    {
      PrintString((char *) "Servos at 127 (neutral) ");
      PrintCR();
      toggle = 2;
    }
  } 
  else if (!rc_dig_in09)
  { /* Force servos to 254 - far left */
    LEFT_TRACKER_SERVO  = 254;
	  RIGHT_TRACKER_SERVO = 254;
    if (toggle != 1)
    {
      PrintString((char *) "Servos at 254");
      PrintCR();
      toggle = 1;
    }
  } 
  else if (!rc_dig_in10)
  { /* Force servos to 0 - far right */
    LEFT_TRACKER_SERVO  = 0;
	  RIGHT_TRACKER_SERVO = 0;
    if (toggle != 0)
    {
      PrintString((char *) "Servos at 0");
      PrintCR();
      toggle = 0;
    }
  } 
  else
  {
    toggle = -1;
    IR_Sensor_Tracking();
  }
}

/*******************************************************************************
*
*	FUNCTION:		IR_Sensor_Tracking()
*
*	PURPOSE:		Run the IR tracker
* 
*	CALLED FROM:	Process_Data_From_Local_IO
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*******************************************************************************/
void IR_Sensor_Tracking(void)
{
	static unsigned int Old_Clock = 0;
  static unsigned char Status1=4, Status2=4;

	if (Clock > Old_Clock) // stay synchronized to beacon data updates (this is important)
	{
  	Old_Clock = Clock; // take a snapshot of the clock

		Track_Beacon(LEFT);  // allow the left beacon tracker state machine to run
		Track_Beacon(RIGHT); // allow the right beacon tracker state machine to run

  	// Update the servo motor positions
	  LEFT_TRACKER_SERVO = Tracker_Data[LEFT].Position;
  	RIGHT_TRACKER_SERVO = Tracker_Data[RIGHT].Position;

   /*** Display Tracker Status ***/
      
    if ((Status1 != Tracker_Data[LEFT].Status) || (Status2 != Tracker_Data[RIGHT].Status))
    {
      Status1 = Tracker_Data[LEFT].Status;
      Status2 = Tracker_Data[RIGHT].Status;

      /** Display current status **/
      /* Tracker Status: 0=None, 1=Only right, 2=Only Left, 3=Both */
      PrintString((char *) "Tracker LEFT: ");
      switch (Tracker_Data[LEFT].Status)
      {
        case 0: PrintString((char *) "None ");
                break;
        case 1: PrintString((char *) "Right");
                break;
        case 2: PrintString((char *) "Left ");
                break;
        case 3: PrintString((char *) "Both ");
                break;
        default: PrintString((char *) "FAULT");
      }
      PrintString((char *) ",     Tracker RIGHT: ");
      switch (Tracker_Data[RIGHT].Status)
      {
        case 0: PrintString((char *) "None ");
                break;
        case 1: PrintString((char *) "Right");
                break;
        case 2: PrintString((char *) "Left ");
                break;
        case 3: PrintString((char *) "Both ");
                break;
        default: PrintString((char *) "FAULT");
      }
      PrintCR();
    }

    //if ((Tracker_Data[LEFT].Status  == BOTH_IN_VIEW) &&
    //	  (Tracker_Data[RIGHT].Status == BOTH_IN_VIEW))
		//{
			// both trackers have an angular solution; calculate beacon vector
			// here using the known baseline distance between the trackers and
			// angles derived from the two Tracker_Data[].Position variables

			// calculate new drive motor PWM values here
	//	}

	}
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
